;;; rx-test.el

(defconst rx-test-patterns
  '(
    (rx (not word))
    (rx (not digit))
    (rx (and "\n" (not blank)))

    (rx (any "^"))
    (rx (not (any "^")))
    (rx (any ?^ ?a))
    (rx (any ?a ?^))

    (rx (any ?- ?a ?\]))
    (rx (any ?a ?- ?\]))
    (rx (any ?A ?- ?\]))
    (rx (any "--]"))
    (rx (any "-a]"))

    (rx (any "."))
    (rx (not (any ".")))
    (rx (not (not (any "."))))

    (rx not-wordchar)
    (rx (any not-wordchar))
    (rx (any ascii not-wordchar))

    (rx (any symbol-start symbol-end))
    (rx (any word-boundary))

    (rx (not (not word-boundary)))
    (rx (not (not (syntax word))))
    (rx (not wordchar))
    (rx (not (not wordchar)))
    (rx (not (not-wordchar)))

    (rx (or (and "a" "b") (and "b" "c")))
    (rx (or (and "a" "b") (and "a" "c")))

    (rx anything)
    (rx anything "abc")
    (rx (or anything "abc"))
    (rx (* (or anything "abc")))
    (rx (anything))

    (rx-to-string "ab" t)
    (rx-to-string "ab" nil)
    (rx-to-string '(and "a" "b") t)
    (rx-to-string '(and "a" "b") nil)
    (rx-to-string '(or (and "a" "b") (and "b" "c")) t)
    (rx-to-string '(or (and "a" "b") (and "b" "c")) nil)
    (rx-to-string 'anything t)
    (rx-to-string 'anything nil)

    ;; comments in rx.el
    (rx (and line-start (0+ (in "a-z"))))
    (rx (and "\n" (not blank)))
    (rx (and "\n" (not (any " \t"))))
    (rx "*** EOOH ***\n")
    (rx (and word-start (submatch (or "catch" "finally")) word-end
	     (not (any ?_))))
    (rx (and (zero-or-more (in " \t\n")) ":"
	     (submatch (or line-end (one-or-more (not (any ?:)))))))
    (rx (and line-start "content-transfer-encoding:" (+ (? ?\n)) blank
	     "quoted-printable" (+ (? ?\n)) blank))
;;;    (rx (and line-start (eval something-else)))
;;;    (rx-to-string '(and line-start ,something-else)), dynamically.
;;;    (rx (or STRING1 STRING2 ...))
    (rx (or (and line-start ";;" (0+ space) ?\n)
	    (and line-start ?\n)))
    (rx (and "$Id: " (1+ (not (in " "))) " "
	     (submatch (1+ (not (in " ")))) " "))
    (rx (and ?\\ ?\\ ?\[ (1+ word)))
    ))


(defun rx-test-execute ()
  (interactive)
  (let ((print-escape-nonascii t)
	(print-escape-newlines t)
	(print-quoted t)
	r1 r2)
    (load-library "rx")
    (setq r1 (mapcar (lambda (s)
		       (condition-case v
			   (eval s)
			 (error v)))
		     rx-test-patterns))
    (load-library "rx-new")
    (setq r2 (mapcar (lambda (s)
		       (condition-case v
			   (eval s)
			 (error v)))
		     rx-test-patterns))
    (with-output-to-temp-buffer "*RX*"
      (with-current-buffer standard-output
	(mapc (lambda (s)
		(insert (my-prin1-to-string s))
		(if (< 40 (current-column)) (insert "\n"))
		(indent-to 40)
		(insert (format "=> %s\n" (my-prin1-to-string (pop r1))))
		(indent-to 40)
		(insert (format "=> %s\n" (my-prin1-to-string (pop r2)))))
	      rx-test-patterns)
	(setq truncate-lines t)))))


(fset 'my-prin1-to-string 'prin1-to-string)
